#!/bin/bash
#############################################################################
#
# Copyright (c) 2011, Dell Inc. All Rights Reserved.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Dell Oracle Deployment Utility
#
# Adam Miller - a_j_miller@dell.com
# Wesley Vaske = wesley_vaske@dell.com
# Collin Murphy = collin_m@dell.com
# Copyright (C) 2010-2013 Dell Inc.
#
# This utility is written in order to supplement the Oracle Validated RPM
# configurations as are distributed by Oracle Inc.
#
# NOTE: We have forked the Oracle Validated RPM as it has a hard requirement on
# Oracle UEK Kernel platform and we want to keep Red Hat and Oracle support.
# Dell Validated RPM is now the dependency.
#
# Here you will find Dell Value Adds and implementations of best practices
# configurations that are created to make the lives easier of those who are
# deploying Oracle DB in an array of configurations upon Dell Solutions.
#
# This script is written and tested for Bash version 3.0 and above
#
# Script version = $GBL_dodeploy_version
#############################################################################


# Define global variables, prefixed with GBL_ to note they are global
GBL_grid_infra=""
GBL_ora_ver="" # This will hold user defined oracle version
GBL_ora_supported_ver=("12cR2")
GBL_group_id="54325"
GBL_disable_selinux=""
GBL_dodeploy_version="0.3"
GBL_quiet_install="false"

function fn_usage {
    printf "Dell Oracle Deployment Utility\n"
    printf "Usage: dodeploy [OPTION]\n"
    printf "\t-h\t\t\tDisplay this help dialog.\n"
    printf "\t-r [ORA_REL]\t\tOracle Release, Valid options: ${GBL_ora_supported_ver[*]}\n"
    printf "\t-g\t\t\tSetup environment for Grid Infrastructure.\n"
    printf "\t-s\t\t\tDisable SELinux.\n"
    printf "\t-v\t\t\tPrint Version and exit.\n"
    printf "\t-q\t\t\tQuiet install. Use default options.\n"
    printf "\t-d\t\t\tUninstall\n"
    exit 2
}

function fn_check_oracle_release {
    for i in ${GBL_ora_supported_ver[@]}; do
        if [[ "$GBL_ora_ver" == "$i" ]]; then
            return 0
        fi
    done
    
    return 1
}

function fn_add_group {
    # Add groups required for Oracle DB
    group_id=""

    if [[ -n $1 ]] && [[ -n $2 ]]; then
        group_name=$1
        group_id="-g $2"
    else
        # Error: not enough parameters passed, return the fail
        return 1
    fi
    
    printf "Trying to add new group: $group_name ...\n"
    grep "^$group_name:" /etc/group > /dev/null 2>&1

    if [[ $? -ne 0 ]]; then
        # Add the group since it does not exist
        groupadd $group_id $group_name > /dev/null 2>&1
        printf "\tAdded group: $group_name with id: $2\n" 
    else
        printf "\tgroup: $group_name - already exists\n"
    fi

}


# FIXME - do we want to disable SELinux anymore?
function fn_disable_selinux {
    selinux_config="/etc/selinux/config"
    selinux_config_bkup=$selinux_config.bkup

    test ! -e "$selinux_config_bkup" && cp $selinux_config $selinux_config_bkup
    printf  "Disabling SELINUX, if not already disabled in $selinux_config file\n"
    selinux_val=$( grep "^SELINUX=" $selinux_config | cut -d"=" -f2 | 
                                                        sed 's/[ \t]*$//' )
    if [[ -z "$selinux_val" ]]; then
        printf "SELINUX=disabled\n" >> ${selinux_config}
        return 0  
    fi
    
    if [[ "$selinux_val" != "disabled" ]]; then
        test -e $selinux_config.tmp && rm -f $selinux_config.tmp
        touch $selinux_config.tmp
        while read line
        do
            printf "$line\n" | grep -e "^#" > /dev/null
            if [[ $? == 0 ]]; then
                printf "$line\n" >> $selinux_config.tmp
            elif [[ "$line" =~ "SELINUX=" ]]; then
                printf "SELINUX=disabled\n" >> $selinux_config.tmp
            else
                printf "$line\n" >> $selinux_config.tmp
            fi
            
        done < $selinux_config
        mv ${selinux_config}.tmp ${selinux_config}
        return 0

    else
        return 1
    fi
}



function fn_setup_grid_infrastructure {

    # We can assume the oracle user exists because the RPM package containing 
    # this utility depends on the Oracle Preinstall RPM which creates that user
    # for us.
    
    # Create necessary groups

    printf "Creating groups required for Oracle Database with Grid Infrastructure...\n" 
    
    # Oracle Preinstall RPM should have provided the oinstall and dba groups 
    # already so we need not concern ourselves with those 
    
    if [[ "$GBL_ora_ver" == "12cR2" ]]; then
        groupnum=$(cat /etc/group | sort -t: -g +2 -3 | grep -v nfsnobody | cut -f3 -d":" | tail -1)
        
        if [[ "$groupnum" -ge "$GBL_group_id" ]]; then
            GBL_group_id=$(( $groupnum + 1 ))
        fi
	
        fn_add_group asmadmin $GBL_group_id
        fn_add_group asmdba $(( $GBL_group_id + 1 ))
        fn_add_group asmoper $(( $GBL_group_id + 2 ))
	fn_add_group backupdba $(( $GBL_group_id + 3 ))
	fn_add_group dgdba $(( $GBL_group_id + 4 ))
	fn_add_group kmdba $(( $GBL_group_id + 5 ))
  
	printf "Created Job Separation Groups:\n"
	printf "\t- asmadmin, asmdba, asmoper\n"
	printf "\t- backupdba, dgdba, kmdba\n"
		
        printf "Creating grid user...\n"
        grep "^grid:" /etc/passwd   > /dev/null 2>&1

		
        if [[ $? -ne 0 ]]; then
            
            useradd -g oinstall -G asmadmin,asmoper,asmdba,dba grid  > /dev/null 2>&1
            #printf "oracle" | passwd --stdin grid
            printf "Added user: grid\n"
            printf "\t- primary group: oinstall\n"
            printf "\t- secondary group(s): dba,asmdba,asmadmin,asmoper\n" 
	else
            printf "user: grid - already exists\n"
        fi
		
		
        printf "grid\tsoft\tnofile\t1024\n" >> /etc/security/limits.conf
	printf "grid\thard\tnofile\t65536\n" >> /etc/security/limits.conf
	printf "grid\tsoft\tnproc\t16384\n" >> /etc/security/limits.conf
	printf "grid\thard\tnproc\t16384\n" >> /etc/security/limits.conf
	printf "grid\tsoft\tstack\t10240\n" >> /etc/security/limits.conf
	printf "grid\thard\tstack\t32768\n" >> /etc/security/limits.conf
        printf "\t- pam limits set\n"


        if id oracle | grep "(asmoper)" > /dev/null 2>&1; then
            printf "Oracle user already a member of asmoper group.\n"
        else
            printf "Adding Oracle user to asmoper group... "
            usermod -a -G asmoper,asmdba oracle
            printf "Done\n"
        fi

	usermod -a -G backupdba,dgdba,kmdba oracle > /dev/null 2>&1
	printf "Added user Oracle to Job Separation Groups:\n"
	printf "\t- backupdba, dgdba, kmdba\n"

        printf "Modifying kernel parameters in sysctl.conf:\n"
        printf "##dell-oracle-rdbms-utilities setting for Oracle DB\n" >> /etc/sysctl.conf
        printf "vm.swappiness = 1\n" >> /etc/sysctl.conf
        printf "vm.dirty_background_ratio = 3\n" >> /etc/sysctl.conf
        printf "vm.dirty_ratio = 80\n" >> /etc/sysctl.conf
        printf "vm.dirty_expire_centisecs = 500\n" >> /etc/sysctl.conf
        printf "vm.dirty_writeback_centisecs = 100\n" >> /etc/sysctl.conf
        printf "fs.aio-max-nr = 5281156\n" >> /etc/sysctl.conf
        printf "Applying kernel parameters persistent across reboots:"
        sysctl -p  > /dev/null 2>&1

        # Disabling avahi-daemon service as part of grid/oracle DB preinstall"
        printf "stop/disable avahi-daemon service"
        systemctl stop avahi-daemon  > /dev/null 2>&1
        systemctl disable avahi-daemon  > /dev/null 2>&1

        # Setting RemoveIPC=no in /etc/systemd/logind.conf to overcome ORA-27300/27301/27302 DB Crash Errors
        printf "Setting RemoveIPC=no in /etc/systemd/logind.conf"
        printf "RemoveIPC=no" >> /etc/systemd/logind.conf
        systemctl daemon-reload > /dev/null 2>&1
        systemctl restart systemd-logind > /dev/null 2>&1

        # Need to perform directory setup and permissions
        if [[ "$GBL_ora_ver" == "12cR2" ]]; then
            fn_setup_grid_dir "12.2.0" ; # Function call
        fi
    fi

    if grep '6.' /etc/redhat-release; then

        ## NOTE: This is specific to RHEL6 because Upstart replaced the legacy 
        ##       SysV init system and therefore the line from the grid installer
        ##       that is placed in /etc/inittab is no longer applicable.
        printf "Setting up ohasd Upstart configuration in: /etc/init/dell-ora.conf\n"
        cat << EOF
# Dell Oracle Grid Infrastructure ohasd config
#
# As of Oracle 11gR2 there is the following line placed in /etc/inittab but is
# no longer in use as of RHEL6 because of the switch from SysV init to Upstart
#
# h1:35:respawn:/etc/init.d/init.ohasd run >/dev/null 2>&1 </dev/null
#
# The following configuration is simply the migration of that line to the new
# syntax

start on stopped rc RUNLEVEL=[35]
stop on runlevel [S01246]

respawn
script 
    /etc/init.d/init.ohasd run >/dev/null 2>&1 </dev/null
end script
EOF
fi

}

###Already done in oradb.py####redo here and undo in orab.py
#Setup the ORACLE_BASE directory for installation with correct perms
function fn_setup_oracle_dir {

    printf "Setting up Ownerships and Permissions for Oracle Directory... \n"
    
    # Provide the user a default but replace with user provided if necessary
    oracle_base="/u01/app/oracle"
    if [[ "$GBL_quiet_install" == "false" ]]; then
        printf "Enter ORACLE_BASE path/partition [$oracle_base]: "
        read in_oracle_base
    fi
    
    # If the user offers a value, use it instead
    if [[ -n "$in_oracle_base" ]]; then
        oracle_base="$in_oracle_base"
    fi
    
    # Create the directory tree if necessary
    if [[ ! -d "$oracle_base" ]]; then
        mkdir -p $oracle_base
    fi

    # # Apply appropriate ownership
    # chown -R oracle.oinstall $oracle_base
    
    
    # # Get oracle root dir. If default, root dir is /u01 
    # ora_root_dir="/$(printf "$oracle_base" | cut -d/ -f2)"
    # chmod -R 775 $ora_root_dir


    #create Oracle Database (RDBMS) directories
    oracle_RDBMS="/u01/app/oracle/product/12.2.0/dbhome_1"

    # Create the directory tree if necessary
    if [[ ! -d "$oracle_RDBMS" ]]; then
        mkdir -p $oracle_RDBMS
    fi

    # Apply appropriate ownership
    chown -R oracle.oinstall $oracle_base

    # Get oracle root dir. If default, root dir is /u01 
    ora_root_dir="/$(printf "$oracle_base" | cut -d/ -f2)"
    chmod -R 775 $ora_root_dir

    printf "DONE\n"

}

# function fn_setup_grid_dir
#
# @param - ora_version
# 
# This function will setup the grid directories needed, passing in the oracle
# release version as the parameter to be used for the creation of sections
# of the directory structure.
function fn_setup_grid_dir {

    # Since there are more than one version of oracle that can have grid
    # setup, we need to read that parameter in and error if not provided
    if [[ -z "$1" ]]; then
        return 1
    fi
    
    ora_version="$1"


    printf "Setting up Ownerships and Permissions for Grid Directory... \n"

    # Provide the user a default but replace with user provided if necessary
    grid_base="/u01/app/grid"
    if [[ "$GBL_quiet_install" == "false" ]]; then
        printf "Enter Oracle Grid installation base path/partition [$grid_base]: "
        read in_grid_base
    fi
    
    # If the user offers a value, use it instead
    if [[ -n "$in_grid_base" ]]; then
        grid_base="$in_grid_base"
    fi
   
    # Create the directory tree if necessary
    if [[ ! -d "${grid_base}" ]]; then
        mkdir -p $grid_base
    fi

    # Provide the user a default but replace with user provided if necessary
    grid_home="/u01/app/$ora_version/grid"
    if [[ "$GBL_quiet_install" == "false" ]]; then
        printf "Enter Oracle Grid Home path [$grid_home]: "
        read in_grid_home
    fi
    
    # If the user offers a value, use it instead
    if [[ -n "$in_grid_home" ]]; then
        grid_base="$in_grid_home"
    fi
   
    # Create the directory tree if necessary
    if [[ ! -d "$grid_home" ]]; then
        mkdir -p $grid_home
    fi

    # Get the grid root dir. If default, root dir is /u01   
    grid_root_dir="/$(printf "$grid_base" | cut -d/ -f2)"
    
    # Apply appropriate ownership
    chown -R grid.oinstall $grid_root_dir
}

#############################################################################

# Main script calls

# Check to make sure we're running as root.
if [[ $(id -u) -ne 0 ]]; then
    printf "ERROR: Script must be run as root\n"
    exit 1
fi

# Check for parameters, we require at least one.
if [[ $# -lt 1 ]]; then
    fn_usage ;
    exit 0
fi

# Obtain all of our opts from the command line
while getopts ":hVgsqr:d:" opt; do
    case $opt in
        h)
            fn_usage ;
            exit 0
            ;;
        g)
            GBL_grid_infra="true"
            ;;
        s)
            GBL_disable_selinux="true"
            ;;
        r)
            GBL_ora_ver="$OPTARG"            
            ;;
        V)
            printf "dodeploy version: $GBL_dodeploy_version\n"
            exit 0
            ;;
        q)
            printf "Quiet install. Use default options.\n"
            GBL_quiet_install="true"
            ;;
        d)
            printf "Uninstalling"
            exit 0
            ;;
        *)
            fn_usage ;
            exit 101
            ;;
    esac
done

fn_check_oracle_release ; # Function call to check for valid Oracle Release
if [[ "$?" -eq "1" ]]; then
    printf "ERROR: Invalid Oracle Release version provided or none specified.\n"
    printf "Supported Releases: ${GBL_ora_supported_ver[*]}\n"
    exit 3
fi

if [[ "$GBL_grid_infra" == "true" ]]; then
    # We need to perform this outside the getopts loop so we can obtain all
    # the opts before launching the function.
    fn_setup_grid_infrastructure ; # Function call
fi

# Setup Oracle directory and permissions #########
######Already in oradb.py
## This needs to run *after* the grid setup so that the permissions are correct
## since grid owns everything else in (default) /u01/app/
fn_setup_oracle_dir ; # Function call

if [[ "$GBL_disable_selinux" == "true" ]]; then
    fn_disable_selinux ;
fi
